/**	
*   @file	OLine.cpp
*	@author	Mathieu Delalandre, Sebastien Perin, Omar Hamidou
*	@date	05/2004	
*/

#include <iostream>
#include <string>
#include <math.h>

using namespace std;

#include <GOMLib\OGraphic.hpp>
#include <GOMLib\OPoint.hpp>
#include <GOMLib\PMath.hpp>

#include <GOMLib\OLine.hpp>


/**
*  Constructor
*/
OLine::OLine()
{
	int a=1;
}


/**
*  Constructor
*/
OLine::OLine(OPoint &  b, OPoint & e)
{
	this->b = b;
	this->e = e;
}


/**
*  Constructor
*/
OLine::OLine(int x1, int y1,int x2, int y2)
{
	b.set(x1,y1);
	e.set(x2,y2);
}


/**
*  Destructor
*/
OLine::~OLine()
{
}


/**
*  Get the end(x)-begin(x) delta x
*/
int OLine::getDX()
	{return e.getX()-b.getX();} 

	
/**
*  Get the end(y)-begin(y) delta y
*/
int OLine::getDY()
	{return e.getY()-b.getY();} 

	
/**
*  Get length of line
*/
double OLine::getLength()
{ 
	return e.euclid(b);
}


/**
*  Match length (0 to inf):
*  (lmax - lmin)/lmin
*  if equals return '0', else a distance
*  bijective: l1.matchLength(l2) == l2.matchLength(l1)
*/
double OLine::matchLength(OLine & l)
{ 
	double l1,l2;
	l1 = this->getLength();
	l2 = l.getLength();
	if(l1>=l2)
		return (l1-l2)/l2;
	return (l2-l1)/l1;
}


/**
*  The lines are opposite,
*  the line1's begin is near of line2's end
*  and
*  the line1's end is near of line2's begin
*/
bool OLine::isOpposite(OLine & l)
{
	double l1=0,l2=0;
	l1 = this->getBegin().euclid(l.getBegin());
	l2 = this->getEnd().euclid(l.getEnd());
	
	double l3=0,l4=0;
	l3 = this->getBegin().euclid(l.getEnd());
	l4 = this->getEnd().euclid(l.getBegin());

	if((l1+l2)<=(l3+l4))
		return false;
	return true;
}


/**
*  Move the line with dx,dy
*  mode 1 move begin and end, 2 begin, 3 end
*/
void OLine::move(int dx,int dy,int mode)
{ 
	switch(mode)
		{
		case 1:
			b.move(dx,dy);
			e.move(dx,dy);
			break;
		case 2:
			b.move(dx,dy);
			break;
		case 3:
			e.move(dx,dy);
			break;
		}
}


/**
*  Move the line to center (begin to 0,0)
*/
void OLine::movec()
{ 
	OPoint p = b;
	b = b - b;
	e = e - p;
}


/**
*  Reverse the line 
*  (end to begin and begin to end)
*/
void OLine::reverse()
{ 
	OPoint p = b;
	b = e;
	e = p;
}


/**
*  Reverse and return a new line 
*  (end to begin and begin to end)
*/
OLine OLine::getReverse()
{ 
	OLine rl(e,b);
	return rl;
}


/**
*  Return the line's middle
*/
OPoint OLine::getMiddle()
{ 
	return (b + e)/2;	
}


/**
*  Test if lines are connected,
*  they are a same point (begin or end)
*/
bool OLine::isConnected(OLine & l)
{ 
	if	(
		(this->getBegin() == l.getBegin())
		||
		(this->getBegin() == l.getEnd())
		||
		(this->getEnd() == l.getBegin())
		||
		(this->getEnd() == l.getEnd())
		)
		return true;
	return false;
}


/**
*  Test if lines are near,
*  they are a nearest point (begin or end)
*/
bool OLine::isNear(OLine & l)
{ 
	if	(
		(this->getBegin().isConnected(l.getBegin()))
		||
		(this->getBegin().isConnected(l.getEnd()))
		||
		(this->getEnd().isConnected(l.getBegin()))
		||
		(this->getEnd().isConnected(l.getEnd()))
		)
		return true;
	return false;
}


/**
*  Get tangent of line (y2-y1)/(x2-x1)
*/
double OLine::getTangent()
{ 
	if(getDX() == 0)
		return 0;
	return ((double)getDY()/(double)getDX());
}


/**
*  Get origin
*/
int OLine::getOrigin()
{ 
	return nearest(b.getY() - this->getTangent()*b.getX());
}


/**
*  Get line's direction in radian,
*  the arctangent of (y2-y1)/(x2-x1)
*/
double OLine::getDirection()
{ 
	return atan2(getDY(),getDX());
}


/**
*  Get direct angle between the line 
*  and a end connected line.
*  This direct angle is always lower than pi.
*  example:  
*	OLine l1(10,10,20,10);
*	OLine l2(20,10,20,20);
*	OLine l3(20,10,20,0);
*	cout << l1.getDAngle(l2); -> pi/2
*	cout << l1.getDAngle(l3); -> pi/2
*/
double OLine::getDAngle(OLine & l2)
{ 
	if(e != l2.getBegin())
		return -1;
	
	OLine l3(b,l2.getEnd());
	double a,b,c;
	a = getLength();
	b = l2.getLength();
	c = l3.getLength();
	double r = acos((pow(a,2) + pow(b,2) - pow(c,2))/(2*a*b));
	
	return r;
}


/**
*  Get angle between the line 
*  and a end connected line.
*  This angle is inclued between 0 and 2*pi.
*  example:  
*	OLine l1(10,10,20,10);
*	OLine l2(20,10,20,20);
*	OLine l3(20,10,20,0);
*	cout << l1.getAngle(l2); -> pi/2
*	cout << l1.getAngle(l3); -> 3*pi/2
*/
double OLine::getAngle(OLine & l)
{ 
	if(e != l.getBegin())
		return -1;
	
	OLine lc = l; lc.movec();
	double d = getDirection();
	OPoint oc = lc.getEnd(); oc.scalar(d);

	double a = getDAngle(l);
	if(oc.getY()>0)
		return a;
	else
		return 2*pi - a;
}


/**
*  Get North azimuth
*/
double OLine::getNAzimuth()
{ 
	OLine hl(b.getX(),b.getY(),b.getX(),b.getY()-10);
	OLine rl = *this; rl.reverse();
	return rl.getAngle(hl);
}


/**
*  Get South azimuth
*/
double OLine::getSAzimuth()
{ 
	OLine hl(b.getX(),b.getY(),b.getX(),b.getY()+10);
	OLine rl = *this; rl.reverse();
	return rl.getAngle(hl);
}


/**
*  Get East azimuth
*/
double OLine::getEAzimuth()
{ 
	OLine hl(b.getX(),b.getY(),b.getX()-10,b.getY());
	OLine rl = *this; rl.reverse();
	return rl.getAngle(hl);
}


/**
*  Get Weast azimuth
*/
double OLine::getWAzimuth()
{ 
	OLine hl(b.getX(),b.getY(),b.getX()+10,b.getY());
	OLine rl = *this; rl.reverse();
	return rl.getAngle(hl);
}


/**
*  The lines are parallel 
*/
bool OLine::isParallel(OLine & l)
{
	if((this->getDX()*l.getDY() - this->getDY()*l.getDX()) == 0)
		return true;
	return false;
}


/**
*	Match Teta (0 to pi/2),
*   Teta is the parallelism gap between two lines:
*   if parallel return '0', else a distance until '1':
*     Teta/(pi/2)
*   bijective: l1.matchTeta(l2) == l2.matchTeta(l1)
*/
double OLine::matchTeta(OLine &l)
{
	double d1 = this->getDirection();
	if(d1<0) d1 += pi;
	
	double d2 = l.getDirection();
	if(d2<0) d2 += pi;	
	
	double teta1 = fabs(d1-d2);
	double teta2 = pi-teta1;
	if(teta2<teta1)
		return teta2;
	return teta1;
}


/**
*	The lines are orthogonal
*/
bool OLine::isOrthogonal(OLine & l)
{
	if((this->getDX()*l.getDX() + this->getDY()*l.getDY()) == 0)
		return true;
	return false;
}


/**
*  Is on 
*  test if a point corresponds to line equation y=m*x + p
*/
bool OLine::isColinear(OPoint & p)
{
	double ay = this->getTangent()*p.getX() + this->getOrigin();
	if(p.getY() == nearest(ay))
		return true;
	return false;
}


/**
*  The point is between line's begin and line's end 
*  dx1 dx2 are opposite or dy1 dy2 must be opposite
*  This method isn't bijective
*/
bool OLine::between(OPoint & p)
{
	int dx1,dx2,dy1,dy2;
	dx1 = p.getDX(b); dy1 = p.getDY(b);
	dx2 = p.getDX(e); dy2 = p.getDY(e);
	if(((dx1==0)&&(dy2==0)) || ((dx2==0)&&(dy1==0))) { return true; }
	if((dx1*dx2 < 0)||(dy1*dy2 < 0))
		return true;
	return false;
}


/**
*  Test with OLine::isColinear() and  OLine::between()
*  in these case the point belongs to line object
*/
bool OLine::belong(OPoint & p)
{
	if(this->isColinear(p))
		if(this->between(p))
			return true;
	return false;
}


/**
*  Get colinear intersection between two line objects
*/
OPoint OLine::getIntersection(OLine & l)
{	
	OPoint pi;
	if(!this->isParallel(l))
		{
		double m1,p1,m2,p2;
		m1 = this->getTangent(); p1 = this->getOrigin();
		m2 = l.getTangent(); p2 = l.getOrigin();
		int xi = nearest((p2-p1)/(m1-m2));
		int yi = nearest((m2*p1-p2*m1)/(m2-m1));
		if((xi>=0)&&(yi>=0))
			pi.set(xi,yi);
		}
	return pi;
}


/**
*  The lines intersect themself
*/
bool OLine::intersect(OLine & l)
{
	OPoint pi = this->getIntersection(l);
	if(pi.isValid())
		if(this->belong(pi))
			return true;
	return false;
}


/**
*  The lines overlap
*/
bool OLine::overlap(OLine & l)
{
	if	(
		(this->between(l.getBegin()))
		||
		(this->between(l.getEnd()))
		)
		return true;
	return false;
}


/**
*  Get Euclidien distance between the line object and a given point
*/
double OLine::euclid(OPoint & p)
{
	double num = abs(p.getY() - (this->getTangent()*p.getX()) - this->getOrigin());
	double den = sqrt(1+pow(getTangent(),2));
	return(num/den);
}


/**
*  Get Euclidien distance between two line objects,
*  this distance is computed between middle points
*/
double OLine::euclid(OLine & l)
{
	OPoint m1 = this->getMiddle();
	OPoint m2 = l.getMiddle();
	return m1.euclid(m2);
}


/**
*  == operator
*/
bool OLine::operator == (OLine & l)
{
	if((b == l.getBegin())&&(e == l.getEnd()))
		return true;
	return false;
}


/**
*  != operator
*/
bool OLine::operator != (OLine & l)
{
	if((b != l.getBegin())||(e != l.getEnd()))
		return true;
	return false;
}


/**
*  / operator
*/
OLine OLine::operator / (int v)
{
	OPoint se = e/v;
	OLine r(b,se);
	return r;
}


/**
*  * operator
*/
OLine OLine::operator * (int v)
{
	OPoint se = e*v;
	OLine r(b,se);
	return r;
}


/**
*  GOM output
*/
void OLine::gomo(ostream & os)
{
	os << getName() << endl;
	b.gomo(os);
	e.gomo(os);
}


/**
*  GOM input
*/
void OLine::gomi(istream & is)
{
	string ptn;
	is >> ptn;
	b.gomi(is);
	is >> ptn;
	e.gomi(is);	
}


/**
*  XML output
*/
void OLine::xmlo(ostream & os)
{
	OPoint p = getMiddle();
	os	<< "<" 
		<< getName()
		<< " length=\"" << getLength() << "\""
		<< " direction=\"" << getDirection() << "\""
		<< ">" << endl;
	os	<< "<" << "center x=\"" << p.getX() <<"\" y=\"" << p.getY() << "\"/>" << endl; 
	b.xmlo(os);
	e.xmlo(os);
	os	<< "</" << getName() << ">" << endl;
}


/**
*  SVG output
*/
void OLine::svgo(ostream & os)
{
	if(isValid())
		os	<<	"<line"
			<<	" x1=\"" << b.getX() << "\" y1=\"" << b.getY() << "\""
			<<	" x2=\"" << e.getX() << "\" y2=\"" << e.getY() << "\""
			<<	" style=\"fill:none; stroke:green; stroke-width:1\""
			<<	"/>" << endl ;
}



