/**	
*   @file	OQuadrilateral.cpp
*	@author	Omar Hamidou, Mathieu Delalandre
*	@date	07/2004	
*/


#include <iostream>
#include <string>
#include <algorithm>
#include <math.h>

using namespace std;

#include <GOMLib\OGraphic.hpp>
#include <GOMLib\OPoint.hpp>
#include <GOMLib\OLine.hpp>
#include <GOMLib\PMath.hpp>

#include <GOMLib\OQuadrilateral.hpp>


/**
*	Constructor
*/
OQuadrilateral::OQuadrilateral()
{
}


/**
*	Constructor
*   The parallelogram object must constructed like a bounding box:
*   p1       p2
*   p4       p3
*/
OQuadrilateral::OQuadrilateral(OPoint &  p1, OPoint & p2,OPoint &  p3, OPoint & p4)
{
	this->p1 = p1; 	this->p2 = p2; 	this->p3 = p3; 	this->p4 = p4;
}


/**
*  Constructor
*   The parallelogram object must constructed like a bounding box:
*   begin1     end1
*   end2       begin2
*/
OQuadrilateral::OQuadrilateral(OLine &  l1, OLine & l2)
{
	this->p1 = l1.getBegin(); 
	this->p2 = l1.getEnd(); 
	this->p3 = l2.getBegin(); 
	this->p4 = l2.getEnd(); 
}


/**
*  Destructor
*/
OQuadrilateral::~OQuadrilateral()
{
}


/**
*  Check if parallelogram is a correct model
*  test OLine::isConnected, OLine::isOposite, OLine::intersect
*/
bool OQuadrilateral::check()
{
	OLine l1(p1,p2),l2(p3,p4);
	if(!l1.isConnected(l2))
		if(l1.isOpposite(l2))
			if(!l1.intersect(l2))
				return true;
	return false;
}


/**
* Get middle point
*/
OPoint OQuadrilateral::getMiddle()
{
	return (p1 + p2 + p3 + p4)/4;
}

/**
* return the median axis of a quadrilateral
*/
OLine OQuadrilateral::medianAxis()
{
	OLine l1(p4,p1); OPoint p5 = l1.getMiddle();
	OLine l2(p2,p3); OPoint p6 = l2.getMiddle();
	OLine l(p5,p6);
	return l;
}

/**
* Get the angle between median axis and horizontal axis
*/
double OQuadrilateral::getAngle()
{
	OLine l = medianAxis();
	if(l.getTangent() > 0)
		return l.getWAzimuth();
	else 
		return(l.getWAzimuth()-pi);
}

/**
* Get distance between two quadrilaterals
*/
double OQuadrilateral::getDistance(OQuadrilateral &pa)
{
	double d[4];
	double *ad;
	if(this != &pa)
	{
		d[0] = p2.euclid(pa.p1) + p3.euclid(pa.p4);
		d[1] = p2.euclid(pa.p3) + p3.euclid(pa.p2);
		d[2] = p1.euclid(pa.p2) + p4.euclid(pa.p3);
		d[3] = (pa.p1).euclid(p4) + (pa.p4).euclid(p1);

		if((abs(getAngle()-pa.getAngle())<0.392699))
		{
			ad = min_element(d, d+sizeof(d)/sizeof(d[0]));
			return (*ad);
		}
		else return 1000000000;
	}
	return 1000000000;
}

OLine OQuadrilateral::getUpSide()
{
	OLine l(p1,p2);
	return l;
}

OLine OQuadrilateral::getLeftSide()
{
	OLine l(p4,p1);
	return l;
}

OLine OQuadrilateral::getRightSide()
{
	OLine l(p2,p3);
	return l;
}

OLine OQuadrilateral::getDownSide()
{
	OLine l(p3,p4);
	return l;
}

/**
* Get the center of quadrilateral
*/
OPoint OQuadrilateral::getCenter()
{
	OLine l1(p1,p3);
	OLine l2(p2,p4);
	return(l1.getIntersection(l2));
}

/**
* determine if ther is a commun point between two quadrilaterals
*/ 
bool OQuadrilateral::communPoint(OPoint & p)
{
	if((p1 == p)||(p2 == p)||(p3 == p)||(p4 == p))
		return true;
	return false;
}

/**
* is connected if there are two commun points between quadrilaterals
*/
bool OQuadrilateral::isConnected(OQuadrilateral & q)
{
	int compt = 0;
	if(this->communPoint(q.p1))
		compt++;
	if(this->communPoint(q.p2))
		compt++;
	if(this->communPoint(q.p3))
		compt++;
	if(this->communPoint(q.p4))
		compt++;
	if(compt == 2)
		return true;
	return false;
}

/**
* is touch if there is one commun point
*/
bool OQuadrilateral::isTouch(OQuadrilateral & q)
{
	if(
		(this->communPoint(q.p1))
		||
		(this->communPoint(q.p2))
		||
		(this->communPoint(q.p3))
		||
		(this->communPoint(q.p4))
		)
		return true;
	return false;
}

bool OQuadrilateral::isTouch(OLine & l)
{
	if(
		(this->communPoint(l.getBegin()))
		||
		(this->communPoint(l.getEnd()))
		)
		return true;
	return false;
}


/**
*  == operator
*/
bool OQuadrilateral::operator == (OQuadrilateral & q)
{
	if((p1 == q.p1)&&(p2 == p2)&&(p3 == q.p3)&&(p4 == p4))
		return true;
	return false;
}


/**
*  != operator
*/
bool OQuadrilateral::operator != (OQuadrilateral & q)
{
	if((p1 != q.p1)||(p2 != q.p2)||(p3 != q.p3)||(p4 != q.p4))
		return true;
	return false;
}

/**
*  GOM output
*/
void OQuadrilateral::gomo(ostream & os)
{
	os << getName() << endl;
	p1.gomo(os);
	p2.gomo(os);
	p3.gomo(os);
	p4.gomo(os);
}


/**
*  GOM input
*/
void OQuadrilateral::gomi(istream & is)
{
	string ln;
	is >> ln;
	p1.gomi(is);
	is >> ln;
	p2.gomi(is);	
	is >> ln;
	p3.gomi(is);	
	is >> ln;
	p4.gomi(is);	
}


/**
*  XML output
*/
void OQuadrilateral::xmlo(ostream & os)
{
	c = this->getCenter(); 
	os	<< "<" 
		<< getName()
		<< ">" << endl;
	os	<< "<" << "center x=\"" << c.getX() <<"\" y=\"" << c.getY() << "\"/>" << endl; ;
	p1.xmlo(os);
	p2.xmlo(os);
	p3.xmlo(os);
	p4.xmlo(os);
	os	<< "</" << getName() << ">" << endl;
}


/**
*  SVG output
*/
void OQuadrilateral::svgo(ostream & os)
{
	if(isValid())
		os	<< "<polygon style=\"fill:none; stroke:red; stroke-width:1\"";
		os	<< " points=\""; 
		os	<< p1.getX() << "," << p1.getY() << " ";
		os  << p2.getX() << "," << p2.getY() << " ";
		os  << p3.getX() << "," << p3.getY() << " ";
		os  << p4.getX() << "," << p4.getY();
		os	<<	"\"/>" << endl ;
}