/**	
*   @file	PExample.hpp
*	@author	Mathieu Delalandre
*	@date	06/2004	
*/

/*!	\mainpage GOMLib Documentation
	
	\htmlonly
	<p align="center">
	Mathieu Delalandre<br>
	2004 July<br>
	mathieu.delalandre@univ-rouen.fr
	</p>
	\endhtmlonly

	\par Introduction
	
	The GOMLib "Graphics Object Modelling" is a library 
	for graphical objects modelling and their management.
	This library allows to formalize the graphical knowledge, and their representation
	through a Graphics Object Modelling Language \a "GOML".
	The GOMLib allows too XML and SVG exports of graphics objects.

	\par Install the GOMLib
	To use the GOMLib in your C++ project include the GOMLib.lib and the GTL.lib.
	Be sure to have the GTL.dll in your current directory.
	In your directory source code (cpp and hpp), 
	copy the GOMLib  and GTL header directories. 
	In your main file
	include "GOMLib\GOMLib.hpp" and see the \ref page1 section to use the library.

	\par Use the GOMLib

	\li	\ref page1 \n
	\li	\ref page2 \n
	\li	\ref page3 \n
	\li	\ref page4 \n
	\li	\ref page5 \n
	\li	\ref page6 \n
*/

/*!	\page page1 Getting started
	\dontinclude PExample.cpp

  	\par OGraphic objects use

	This library is based on a generic (and abstract) OGraphic object use.
	From this object, the GOMLib can be easily extended with new object types 
	(see section \ref page6).
	The GOMLib proposes too implementations of standard graphics objects
	(OPoint, OLine, OArc, OCurve, and so on).
	The following function gives an example of objects create and delete:
	
	\skip init1()
	\until //de

	The GOMLib is based on polymorphism, so only OGraphic* are used for objects manipulation.
	You can use the OGraphic::clone() function in order to copy your objects.
	
	\skip copy()
	\until //de
	
	However, you can use the standard object types in your programs.

	\skip init2()
	\until //de

	You can use too directly the standard objects. However, this using create automatic 
	memory allocation, and can generate some errors with the use of container objects
	(see section \ref page2).

	\skip init3()
	\until //de
  
  	\par Read/Write of graphical objects

	The GOMLib represents the graphical knowledge
	through the Graphics Object Modelling Language \a "GOML". 
	The \a GOML files are extended with ".gom".
	Use the different "gom" functions gomRead() and gomWrite() to read/write the "gom" files.
	You can export too in XML and SVG formats with the functions xmlWrite() and svgWrite().
	The following function gives a simple example of arc write and read:
	
	\skip rw()
	\until //de

	The read function imports a OGraphic*, 
	in order to access standard objects' functions, you must:
	
	\li	to know the object type
	\li to cast the OGraphic type

	Use the OGraphic::getName() function and the object dynamic cast functionnality of "GOMLib\POUtility.hpp".
	The following function gives an example of type test and dynamic cast use.
	
	\skip dynamic()
	\until //de

*/

/*!	\page page2 Container objects
	\dontinclude PExample.cpp
	
	This library is based on two container objects, the high level object OHL and the graph object OGraph.
	These two objects allow a pyramid structuring of graphical knowledge.

	\par The OHL object
	
	The OHL object is an object list to store different OGraphic objects.
	The following function gives an example of OHL use to create and write an arc 
	with its center, and radius.

	\skip ohlArc()
	\until //de

	The OHL object contains a OGraphic* array (deque). So, OHL uses only OGraphic types.
	To copy an OHL you must use the OHL::clone() function like this:
	
	\skip ohlCopy()
	\until //de

	In a same way, when you add some objects to an OHL you must clone these objects
	like this: 

	\skip ohlWrite()
	\until //de

	If you want to replace some objects, you can use the OHL::objectPush() and OHL::objectWrite() functions
	like this: 

	\skip ohlPush()
	\until //de

	\par The OGraph object

	The OGraph object is an object to store different OGraphic objects, 
	with structuring links between objects.
	The following function gives an example of OGraph use to create and write an arc 
	with its center, and radius.

	\skip graphArc()
	\until //de

	The OGraph object contains a OGraphic* array (deque). So, OGraph uses only OGraphic types.
	To copy an OGraph you must use the OHL::clone() function like this:
	
	\skip graphCopy()
	\until //de

	In a same way, when you add some objects to an OGraph you must clone these objects
	like this: 

	\skip graphWrite()
	\until //de

	If you want to replace some object, you can use the OGraph::objectPush() and OGraph::objectWrite() functions
	like this: 

	\skip graphPush()
	\until //de
*/

/*!	\page page3 Attribute objects
	\dontinclude PExample.cpp
	
	The GOMLib allows one other generic objects use, the AGraphic object.
	So, it is possible to use container object (see section \ref page2) to define some attributes
	between objects.
	The GOMLib proposes too implementations of standard attribute objects
	(AAngle, ADistance, ALabel, and so on).

  	The following function gives an example of attributes use with an OHL object.

	\skip ohlAttribute()
	\until //de

  	The following function gives an example of attributes use with an OGraph object.

	\skip graphAttribute()
	\until //de
*/

/*!	\page page4 Use-Case
	\dontinclude PExample.cpp
	
	Based on OGraphic objects, AGraphic objects, and container objects (see section \ref page2),
	the GOMLib allows many various modellings of graphical objects.
	So, it is possible to define many knowledge structuring for a given problem.
	We present here a various possible modellings with the GOMLib for the \e linked \e squares \e problem.
	
	\image html ls.jpg

  	The following function gives some modelling examples of linked squares.
	
	\skip modelling()
	\until //de

*/

/*!	\page page5 Knowledge requests
	\dontinclude PExample.cpp
	
	The GOMLib allows various modelling of knowledge. 
	So, in order to exhange this knowledge between algorithms, 
	the container objects allow request functionalities on graphical knowledge.

  	\par OHL requests
	
	The following function gives examples of a read request.  
	\skip ohlRequest1()
	\until //de

  	The following function gives examples of a read/write request.
	\skip ohlRequest2()
	\until //de

  	\par OGraph requests
	
	The following function gives examples of a read request.  
	\skip graphRequest1()
	\until //de

  	The following function gives examples of a read/write request.
	\skip graphRequest2()
	\until //de

*/

/*!	\page page6 Extend the library
	
	to be continues ...
*/