/**	\file	TreatCMatching.hpp
*	\author	Omar Hamidou, Mathieu Delalandre
*	\date	07/04
*/

//include
#include <iostream>
#include <deque>
#include <algorithm>
#include <math.h>

using namespace std;

#include <GTL\graph.h>
#include <GOMLib\GOMLib.hpp>

#include <LGALib\TreatCMatching.hpp>


/***************************************
	Matching class
****************************************/
/**
*  Constructor
*/
Matching::Matching()
	{ 
	l1 = l2 = -1;
	r1 = r2 = r3 = -1;
	}
/**
*  Constructor
*/
Matching::Matching(int l1,int l2,double r1,double r2,double r3)
	{ 
	this->l1 = l1; this->l2 = l2;
	this->r1 = r1; this->r2 = r2; this->r3 = r3;
	}
/**
*  Same matching,
*  test a common match object
*/
bool Matching::sameTwo(Matching & m)
	{
	if	(
		((l1 == m.l1)&&(l2 == m.l2))
		||
		((l1 == m.l2)&&(l2 == m.l1))
		)
		return true;
	return false;
	}
bool Matching::sameOne(Matching & m)
	{
	if	(
		(l1 == m.l1)
		||
		(l1 == m.l2)
		||
		(l2 == m.l1)
		||
		(l2 == m.l2)
		)
		return true;
	return false;
	}
/**
* Is it an upper matching
* It is a fuzzy match method,
* return true if given object is upper-matching,
* false in the other case
*/
bool Matching::isUpperMatching(Matching  & m)
	{
	double f1=0,f2=0,f3=0;
	
	if(r1<m.r1) f1 = 1;
	else if(r1==m.r1) f1 = 0.5;

	if(r2<m.r2) f2 = 1;
	else if(r2==m.r2) f2 = 0.5;
	
	if(r3<m.r3) f3 = 1;
	else if(r3==m.r3) f3 = 0.5;
	
	double f=(f1+f2+f3)/3;
	if(f<0.5)
		return false;
	
	return true;
	}

/***************************************
	CMatch class
****************************************/

/**
* Constructor
*/
CMatch::CMatch()
	{
	th = -1;
	}

/**
* Destructor
*/
CMatch::~CMatch()
	{
	clear();
	}

/**
* Clear
*/
void CMatch::clear()
	{
	la.clear();
	ms.clear();
	pa.clear();
	}


/**
* Match a OHL of OHL object
* be sure to use parse function before
*/
deque<OGraphic*> CMatch::match(deque<OHL*> & hae,double th)
	{
	this->th = th;
	
	OHL *p; deque<OGraphic*> har;
	for(int i=0;i<hae.size();i++)
		{
		if(isLineLoop(hae[i]))
			{
			closeLoop(hae[i]);
			la = castOLineArray(hae[i]->readOGraphicArray()); 
			match();

			p = new OHL();
			p->writeOGraphicArray(pa);
			har.push_back(p);

			clear();
			}
		else
			har.push_back(hae[i]);
		}

	return(har);
	}

/**
* Test if OHL is a line loop,
* is it composed of OLine
* is it large (4 OLine min)
* is it closed (the begin point is near end point (so connected)
*/
bool CMatch::isLineLoop(OHL *o)
	{
	if(o->sameObject((string)"OLine"))
		if(o->osize()>4)
			{
			OPoint b = (castOLine(o->objectRead(0)))->getBegin();
			OPoint e = (castOLine(o->objectRead(o->osize()-1)))->getEnd();
			if(b.isConnected(e))
				return true;
			}
	return false;
	}

/**
* Close the loop, 
* the begin and end point are equals
*/
void CMatch::closeLoop(OHL *o)
	{
	OPoint b = (castOLine(o->objectRead(0)))->getBegin();
	(castOLine(o->objectRead(o->osize()-1)))->setEnd(b);	
	}

/**
* Match an OHL
*/
void CMatch::match()
	{
	initLArray();
	findMatching();
	dualMatchingFilter();
	strokeFilter();
	fuzzySort();
	performMatching();

	unmatched();
	becomeOHL();
	}


/**
* Initialization of the boolean array for the unmatched lines
*/
void CMatch::initLArray()
{
	LTab = new bool[la.size()];
	for(int i=0;i<la.size();i++)
		LTab[i] = false; 
}

/**
* Find all possible matchings
*/
void CMatch::findMatching()
	{
	for(int i=0;i<la.size();i++)
		for(int j=0;j<la.size();j++)
			if(i!=j)
				if(toMatch(*la[i],*la[j]))
					matching(i,j);
	}
	
/**
* Test if two lines can be match,
* they are no connected, opposite, and overlaped
*/
bool CMatch::toMatch(OLine & l1,OLine  & l2) 
	{
	if	(
		(!l1.isConnected(l2))
		&&
		(l1.isOpposite(l2))
		&&
		(l1.overlap(l2))
		)
		return true;
	return false;
	}


/**
* Compute the range matching,
* and crealte a matching object
*/
void CMatch::matching(int i,int j)
	{
	double r1 = gapStroke(*la[i],*la[j]);
	double r2 = la[i]->matchLength(*la[j]);
	double r3 = la[i]->matchTeta(*la[j]);
	ms.push_back(*(new Matching(i,j,r1,r2,r3)));
	}

/**
* Compute a matching between lines mean
* and middles distances,
* so this criterion is a gap search of strocke pattern 'I'
*/
double CMatch::gapStroke(OLine & l1,OLine  & l2)
	{
	double d1 = (l1.getLength() + l2.getLength())/2;
	double d2 = l1.getMiddle().euclid(l2.getMiddle());
	return d2/d1;
	}

/**
* Erase the dual matching objects
*/
void CMatch::dualMatchingFilter()
	{
	for(int i=0;i<ms.size();i++)
		for(int j=i+1;j<ms.size();j++)
			if(ms[j].sameTwo(ms[i]))
				{
				ms.erase(ms.begin()+j);
				j--;
				}
	}

/**
* Stroke filter
*/
void CMatch::strokeFilter()
	{
	for(int i=0;i<ms.size();i++)
		if(ms[i].r1>th)
			{
			ms.erase(ms.begin()+i);
			i--;
			}
	}

/**
* Fuzzy sort of matching objects
*/
void CMatch::fuzzySort()
	{
	deque<Matching> sms;
	
	if(ms.size()>0)
		{
		sms.push_back(ms[0]);
		ms.erase(ms.begin());
		}
	
	int p;
	while(ms.size()!=0)
		{
		p=0;
		while(sms[p].isUpperMatching(ms[0])&&(p!=sms.size()-1))
			p++;
		sms.insert(sms.begin()+p,ms[0]);
		ms.erase(ms.begin());
		}
	ms = sms; sms.clear();
	}

/**
* Perform matching
*/
void CMatch::performMatching()
	{
	for(int i=0;i<ms.size();i++)
		{
		pa.push_back(new OQuadrilateral(*la[ms[i].l1],*la[ms[i].l2]));
		LTab[ms[i].l1] = true;
		LTab[ms[i].l2] = true;
		for(int j=i+1;j<ms.size();j++)
			if(ms[j].sameOne(ms[i]))
				{
				ms.erase(ms.begin()+j);
				j--;
				}
		}
	}

/**
* Look for the vectors which are not matched
*/
void CMatch::unmatched()
{
	pae = new OHL();
	for(int i=0;i<la.size();i++)
	{
		if(LTab[i]==false)
		{
			pae->newObject(la[i]->clone());
			npa.push_back(la[i]->clone());
		}
	}
}

void CMatch::becomeOHL()
{
	OHL *ohl = new OHL(); all = new OHL();
	for(int i=0;i<pa.size();i++)
		npa.push_back(pa[i]->clone());
	ohl->writeOGraphicArray(npa);
	all->newObject(ohl);
}

/**
* Print tab matching
*/
void CMatch::printTabMatching()
	{
	cout << endl;
	for(int i=0;i<ms.size();i++)
		printMatching(i);	
	cout << endl;
	}

/**
* Print tab matching
*/
void CMatch::printMatching(int i)
	{
	cout << ms[i].l1 << ";" << ms[i].l2 << endl;
	cout << ms[i].r1 << ";" << ms[i].r2 << ";" << ms[i].r3 << endl;
	la[ms[i].l1]->print();
	la[ms[i].l2]->print();
	}
